﻿Option Explicit On
Option Strict On

Imports System.Text

Public Class Customer

#Region "Private instance attributes"

    Private m_Id As Guid = Nothing
    Private m_Name As String = String.Empty
    Private m_Address As String = String.Empty
    Private m_JoinDate As Date = Date.MinValue

    ' TLV type constants.
    Private Const CUSTOMER_ID As Byte = 1
    Private Const CUSTOMER_NAME As Byte = 2
    Private Const CUSTOMER_ADDRESS As Byte = 3
    Private Const CUSTOMER_JOIN_DATE As Byte = 4

#End Region

#Region "Public instance attributes"

#Region "Id"

    Public Property Id() As Guid
        Get
            Return m_Id
        End Get
        Set(ByVal value As Guid)
            m_Id = value
        End Set
    End Property

#End Region

#Region "Name"

    Public Property Name() As String
        Get
            Return m_Name
        End Get
        Set(ByVal value As String)
            m_Name = value
        End Set
    End Property

#End Region

#Region "Address"

    Public Property Address() As String
        Get
            Return m_Address
        End Get
        Set(ByVal value As String)
            m_Address = value
        End Set
    End Property

#End Region

#Region "JoinDate"

    Public Property JoinDate() As Date
        Get
            Return m_JoinDate
        End Get
        Set(ByVal value As Date)
            m_JoinDate = value
        End Set
    End Property

#End Region

#End Region

#Region "Constructors"

    ' Default constructor.
    Public Sub New()
        MyBase.New()
    End Sub

    ' Decodes the specified TLV element into a customer entity.
    Public Sub New(ByVal value As Tlv)
        MyBase.New()
        FromTlv(value)
    End Sub

    ' Decodes the specified byte array into a customer entity.
    Public Sub New(ByVal value() As Byte)
        MyBase.New()
        Deserialize(value)
    End Sub

#End Region

#Region "Public instance methods"

#Region "FromTlv"

    Public Sub FromTlv(ByVal value As Tlv)

        If value.Type = ENTITY_CUSTOMER Then

            Dim _Tlvs As TlvCollection = New TlvCollection(value.Value)

            For Each _Tlv As Tlv In _Tlvs
                Select Case _Tlv.Type
                    Case CUSTOMER_ID
                        m_Id = New Guid(_Tlv.Value)
                    Case CUSTOMER_NAME
                        m_Name = Encoding.UTF8.GetString(_Tlv.Value, 0, _Tlv.Length)
                    Case CUSTOMER_ADDRESS
                        m_Address = Encoding.UTF8.GetString(_Tlv.Value, 0, _Tlv.Length)
                    Case CUSTOMER_JOIN_DATE
                        m_JoinDate = New Date(BitConverter.ToInt64(_Tlv.Value, 0)).ToLocalTime
                End Select
            Next

        End If

    End Sub

#End Region

#Region "ToTlv"

    Public Function ToTlv() As Tlv

        Dim _Tlv As Tlv = Nothing
        Dim _Tlvs As TlvCollection = New TlvCollection

        If Not Me.Id.Equals(Guid.Empty) Then _Tlvs.Add(New Tlv(CUSTOMER_ID, m_Id.ToByteArray))
        If Not String.IsNullOrWhiteSpace(m_Name) Then _Tlvs.Add(New Tlv(CUSTOMER_NAME, Encoding.UTF8.GetBytes(m_Name)))
        If Not String.IsNullOrWhiteSpace(m_Address) Then _Tlvs.Add(New Tlv(CUSTOMER_ADDRESS, Encoding.UTF8.GetBytes(m_Address)))
        If Not Date.Compare(m_JoinDate, Date.MinValue) < 0 And Date.Compare(m_JoinDate, Date.MaxValue) > 0 Then _Tlvs.Add(New Tlv(CUSTOMER_JOIN_DATE, BitConverter.GetBytes(m_JoinDate.ToUniversalTime.Ticks)))

        _Tlv = New Tlv(ENTITY_CUSTOMER, _Tlvs.ToByteArray())

        Return _Tlv

    End Function

#End Region

#Region "Deserialize"

    Public Sub Deserialize(ByVal value As Byte())

        Me.FromTlv(New Tlv(value))

    End Sub

#End Region

#Region "Serialize"

    Public Function Serialize() As Byte()

        Return Me.ToTlv.ToByteArray()

    End Function

#End Region

#End Region

End Class
