﻿Option Explicit On
Option Strict On

Imports System.Collections.ObjectModel

Public Class CustomerCollection
    Inherits Collection(Of Customer)

#Region "Constructors"

    ' Default constructor.
    Public Sub New()
        MyBase.New()
    End Sub

    ' Decodes the specified TLV element into a customer collection
    Public Sub New(ByVal value As Tlv)
        MyBase.New()
        FromTlv(value)
    End Sub

    ' Decodes the specified byte array into a customer collection.
    Public Sub New(ByVal value() As Byte)
        MyBase.New()
        Deserialize(value)
    End Sub

#End Region

#Region "Public instance methods"

#Region "FromTlv"

    Public Sub FromTlv(ByVal value As Tlv)

        If value.Type = COLLECTION_CUSTOMERS Then

            Dim _Tlvs As TlvCollection = New TlvCollection(value.Value)

            For Each _Tlv As Tlv In _Tlvs
                If _Tlv.Type = ENTITY_CUSTOMER Then
                    Me.Add(New Customer(_Tlv))
                End If
            Next

        End If

    End Sub

#End Region

#Region "ToTlv"

    Public Function ToTlv() As Tlv

        Dim _Tlv As Tlv = Nothing
        Dim _Tlvs As TlvCollection = Nothing

        If Me.Count > 0 Then
            _Tlvs = New TlvCollection
            For Each _Customer As Customer In Me.Items
                _Tlvs.Add(New Tlv(_Customer.Serialize()))
            Next
            _Tlv = New Tlv(COLLECTION_CUSTOMERS, _Tlvs.ToByteArray)
        End If

        Return _Tlv

    End Function

#End Region

#Region "ToJSON"

    Public Function ToJSON() As String

        Dim _Json As String = String.Empty

        Return _Json

    End Function

#End Region

#Region "Deserialize"

    Public Sub Deserialize(ByVal value As Byte())

        Me.FromTlv(New Tlv(value))

    End Sub

#End Region

#Region "Serialize"

    Public Function Serialize() As Byte()

        Return Me.ToTlv.ToByteArray()

    End Function

#End Region

#End Region

End Class