﻿Option Explicit On
Option Strict On

Public NotInheritable Class Tlv

#Region "Private instance attributes"

    Private m_Type As Byte
    Private m_Value As Byte()

#End Region

#Region "Public instance attributes"

#Region "Type"

    Public Property Type() As Byte
        Get
            Return m_Type
        End Get
        Set(ByVal value As Byte)
            If value >= 0 And value <= 255 Then
                m_Type = value
            End If
        End Set
    End Property

#End Region

#Region "Length"

    Public ReadOnly Property Length() As Integer
        Get
            If m_Value Is Nothing Then
                Return 0
            Else
                Return m_Value.Length
            End If
        End Get
    End Property

#End Region

#Region "Value"

    Public Property Value() As Byte()
        Get
            Return m_Value
        End Get
        Set(ByVal value As Byte())
            m_Value = value
        End Set
    End Property

#End Region

#End Region

#Region "Constructors"

    Public Sub New()
        MyBase.New()
    End Sub

    ' Instantiates a new TLV element from the specified byte array.
    ' Typically used to create a new TLV element from an arbitrary local entity expressed as a byte array.
    Public Sub New(ByVal type As Byte, ByVal value() As Byte)
        MyBase.New()
        m_Type = type
        m_Value = value
    End Sub

    ' Decodes the specified byte array into a TLV element.
    ' Typically used to decode a TLV element received from a remote system.
    Public Sub New(ByVal byteArray() As Byte)
        MyBase.New()
        FromByteArray(byteArray)
    End Sub

#End Region

#Region "Private instance methods"

#Region "FromByteArray"

    Private Sub FromByteArray(ByVal bytes() As Byte)

        Dim _Length As Integer = 0

        m_Type = bytes(0)
        _Length = BitConverter.ToInt32(bytes, 1)
        ReDim m_Value(_Length - 1)

        Array.Copy(bytes, 5, m_Value, 0, _Length)

    End Sub

#End Region

#End Region

#Region "Public instance methods"

#Region "ToByteArray"

    Public Function ToByteArray() As Byte()

        Dim _ByteArray(m_Value.Length - 1 + 5) As Byte     ' Length - 1 + (one byte for type + four bytes for length).
        Dim _LengthAsBytes() As Byte = BitConverter.GetBytes(m_Value.Length)

        _ByteArray(0) = m_Type
        _LengthAsBytes.CopyTo(_ByteArray, 1)
        m_Value.CopyTo(_ByteArray, 5)

        Return _ByteArray

    End Function

#End Region

#End Region

End Class