﻿Option Explicit On
Option Strict On

Imports System.Collections.ObjectModel

Public Class TlvCollection
    Inherits Collection(Of Tlv)

#Region "Constructors"

    ' Default constructor.
    Public Sub New()
        MyBase.New()
    End Sub

    ' Decodes the specified byte array into a TLV sequence.
    Public Sub New(ByVal byteArray() As Byte)
        MyBase.New()
        FromByteArray(byteArray)
    End Sub

#End Region

#Region "Private instance methods"

#Region "ExtractTlv"

    ' Extracts a TVL element from a sequence.
    Private Function ExtractTlv(ByVal byteArray As Byte(), ByVal index As Integer) As Tlv

        Dim _Type As Byte = byteArray(index)
        Dim _Length As Integer = BitConverter.ToInt32(byteArray, index + 1)
        Dim _Value(_Length - 1) As Byte

        Array.Copy(byteArray, index + 5, _Value, 0, _Length)

        Return New Tlv(_Type, _Value)

    End Function

#End Region

#Region "FromByteArray"

    ' Decodes a TLV sequence from a byte array.
    Private Sub FromByteArray(ByVal byteArray() As Byte)

        Dim _BytesRead As Integer = 0

        Do While _BytesRead < byteArray.Length
            Dim _Tlv As Tlv = ExtractTlv(byteArray, _BytesRead)
            _BytesRead += _Tlv.Length + 5
            Me.Add(_Tlv)
        Loop

    End Sub

#End Region

#End Region

#Region "Public instance methods"

#Region "ToTlv"

    ' Encodes a TLV sequence to a byte array.
    Public Function ToByteArray() As Byte()

        Dim _ParentBytes() As Byte = {}

        For Each _ChildTlv As Tlv In Me
            Dim _ChildBytes() As Byte = _ChildTlv.ToByteArray
            Dim _ParentByteIndex As Integer = _ParentBytes.Length
            ReDim Preserve _ParentBytes(_ParentBytes.Length - 1 + _ChildBytes.Length)
            _ChildBytes.CopyTo(_ParentBytes, _ParentByteIndex)
        Next

        Return _ParentBytes

    End Function

#End Region

#End Region

End Class